@@ -16,17 +16,19 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "info/info_memento.h"
 #include "info/info_top_bar.h"
 #include "settings/cloud_password/settings_cloud_password_email_confirm.h"
-#include "settings/settings_chat.h"
-#include "settings/settings_information.h"
-#include "settings/settings_main.h"
-#include "settings/settings_premium.h"
+#include "settings/sections/settings_chat.h"
+#include "settings/sections/settings_information.h"
+#include "settings/sections/settings_main.h"
+#include "settings/sections/settings_premium.h"
+#include "settings/settings_search.h"
 #include "ui/effects/ripple_animation.h" // MaskByDrawer.
 #include "ui/widgets/menu/menu_add_action_callback.h"
+#include "ui/widgets/menu/menu_add_action_callback_factory.h"
+#include "ui/widgets/menu/menu_item_base.h"
 #include "ui/widgets/discrete_sliders.h"
 #include "ui/widgets/buttons.h"
 #include "ui/widgets/shadow.h"
 #include "ui/widgets/popup_menu.h"
-#include "ui/widgets/menu/menu_add_action_callback_factory.h"
 #include "ui/wrap/fade_wrap.h"
 #include "ui/search_field_controller.h"
 #include "ui/ui_utility.h"
@@ -129,12 +131,12 @@ WrapWidget::WrapWidget(
 		) | rpl::flatten_latest() | rpl::distinct_until_changed());
 
 	_wrap.changes(
-	) | rpl::start_with_next([this] {
+	) | rpl::on_next([this] {
 		setupTop();
 		finishShowContent();
 	}, lifetime());
 	selectedListValue(
-	) | rpl::start_with_next([this](SelectedItems &&items) {
+	) | rpl::on_next([this](SelectedItems &&items) {
 		InvokeQueued(this, [this, items = std::move(items)]() mutable {
 			if (_topBar) {
 				_topBar->setSelectedItems(std::move(items));
@@ -145,7 +147,7 @@ WrapWidget::WrapWidget(
 
 	if (const auto topic = _controller->topic()) {
 		topic->destroyed(
-		) | rpl::start_with_next([=] {
+		) | rpl::on_next([=] {
 			if (_wrap.current() == Wrap::Layer) {
 				_controller->parentController()->hideSpecialLayer();
 			} else if (_wrap.current() == Wrap::Narrow) {
@@ -161,15 +163,30 @@ WrapWidget::WrapWidget(
 }
 
 void WrapWidget::setupShortcuts() {
+	const auto isSettings = [=] {
+		return _controller->section().type() == Section::Type::Settings;
+	};
+	const auto isSearchSettings = [=] {
+		return isSettings()
+			&& (_controller->section().settingsType()
+				== ::Settings::Search::Id());
+	};
+
 	Shortcuts::Requests(
 	) | rpl::filter([=] {
-		return requireTopBarSearch()
-			&& (Core::App().activeWindow()
-				== &_controller->parentController()->window());
-	}) | rpl::start_with_next([=](not_null<Shortcuts::Request*> request) {
+		return (Core::App().activeWindow()
+				== &_controller->parentController()->window())
+			&& (requireTopBarSearch() || isSettings());
+	}) | rpl::on_next([=](not_null<Shortcuts::Request*> request) {
 		using Command = Shortcuts::Command;
 		request->check(Command::Search) && request->handle([=] {
-			_topBar->showSearch();
+			if (requireTopBarSearch()) {
+				_topBar->showSearch();
+			} else if (isSearchSettings()) {
+				_content->setInnerFocus();
+			} else if (isSettings()) {
+				_controller->showSettings(::Settings::Search::Id());
+			}
 			return true;
 		});
 	}, lifetime());
@@ -205,7 +222,7 @@ void WrapWidget::startInjectingActivePeerProfiles() {
 		(_1 == Wrap::Side) && _2
 	) | rpl::map(
 		_2
-	) | rpl::start_with_next([this](Dialogs::Key key) {
+	) | rpl::on_next([this](Dialogs::Key key) {
 		injectActiveProfile(key);
 	}, lifetime());
 
@@ -346,14 +363,14 @@ void WrapWidget::createTopBar() {
 		TopBarStyle(wrapValue),
 		std::move(selectedItems));
 	_topBar->selectionActionRequests(
-	) | rpl::start_with_next([=](SelectionAction action) {
+	) | rpl::on_next([=](SelectionAction action) {
 		_content->selectionAction(action);
 	}, _topBar->lifetime());
 
 	if (hasBackButton()) {
 		_topBar->enableBackButton();
 		_topBar->backRequest(
-		) | rpl::start_with_next([=] {
+		) | rpl::on_next([=] {
 			checkBeforeClose([=] { _controller->showBackFromStack(); });
 		}, _topBar->lifetime());
 	} else if (wrapValue == Wrap::Side) {
@@ -365,7 +382,7 @@ void WrapWidget::createTopBar() {
 			_controller->parentController()->closeThirdSection();
 		});
 	}
-	_topBar->storyClicks() | rpl::start_with_next([=] {
+	_topBar->storyClicks() | rpl::on_next([=] {
 		if (const auto peer = _controller->key().peer()) {
 			_controller->parentController()->openPeerStories(peer->id);
 		}
@@ -409,8 +426,16 @@ void WrapWidget::setupTopBarMenuToggle() {
 		addProfileCallsButton();
 	} else if (section.type() == Section::Type::Settings) {
 		addTopBarMenuButton();
-		if (section.settingsType() == ::Settings::Information::Id()
-			|| section.settingsType() == ::Settings::Main::Id()) {
+		if (section.settingsType() == ::Settings::MainId()) {
+			const auto &st = (wrap() == Wrap::Layer)
+				? st::infoLayerTopBarSearch
+				: st::infoTopBarSearch;
+			const auto button = _topBar->addButton(
+				base::make_unique_q<Ui::IconButton>(_topBar, st));
+			button->addClickHandler([=] {
+				_controller->showSettings(::Settings::Search::Id());
+			});
+		} else if (section.settingsType() == ::Settings::InformationId()) {
 			const auto controller = _controller->parentController();
 			const auto self = controller->session().user();
 			if (!self->username().isEmpty()) {
@@ -425,6 +450,7 @@ void WrapWidget::setupTopBarMenuToggle() {
 				});
 			}
 		}
+		setupShortcuts();
 	} else if (key.storiesPeer()
 		&& key.storiesPeer()->isSelf()
 		&& key.storiesAlbumId() != Stories::ArchiveId()) {
@@ -434,7 +460,7 @@ void WrapWidget::setupTopBarMenuToggle() {
 		const auto button = _topBar->addButton(
 			base::make_unique_q<Ui::IconButton>(_topBar, st));
 		button->addClickHandler([=] {
-			_controller->showSettings(::Settings::Information::Id());
+			_controller->showSettings(::Settings::InformationId());
 		});
 	} else if (section.type() == Section::Type::Downloads) {
 		auto &manager = Core::App().downloadManager();
@@ -443,7 +469,7 @@ void WrapWidget::setupTopBarMenuToggle() {
 			manager.loadingListChanges() | rpl::map_to(false),
 			manager.loadedAdded() | rpl::map_to(true),
 			manager.loadedRemoved() | rpl::map_to(false)
-		) | rpl::start_with_next([=, &manager](bool definitelyHas) {
+		) | rpl::on_next([=, &manager](bool definitelyHas) {
 			const auto has = [&] {
 				for ([[maybe_unused]] const auto id : manager.loadingList()) {
 					return true;
@@ -510,7 +536,7 @@ void WrapWidget::addTopBarMenuButton() {
 	Shortcuts::Requests(
 	) | rpl::filter([=] {
 		return (_controller->section().type() == Section::Type::Profile);
-	}) | rpl::start_with_next([=](not_null<Shortcuts::Request*> request) {
+	}) | rpl::on_next([=](not_null<Shortcuts::Request*> request) {
 		using Command = Shortcuts::Command;
 
 		request->check(Command::ShowChatMenu, 1) && request->handle([=] {
@@ -541,7 +567,7 @@ void WrapWidget::addProfileCallsButton() {
 		return user->hasCalls();
 	}) | rpl::take(
 		1
-	) | rpl::start_with_next([=] {
+	) | rpl::on_next([=] {
 		_topBar->addButton(
 			base::make_unique_q<Ui::IconButton>(
 				_topBar,
@@ -549,7 +575,7 @@ void WrapWidget::addProfileCallsButton() {
 					? st::infoLayerTopBarCall
 					: st::infoTopBarCall))
 		)->addClickHandler([=] {
-			Core::App().calls().startOutgoingCall(user, false);
+			Core::App().calls().startOutgoingCall(user, {});
 		});
 	}, _topBar->lifetime());
 
@@ -689,7 +715,7 @@ void WrapWidget::finishShowContent() {
 	_contentChanges.fire({});
 
 	_content->scrollBottomSkipValue(
-	) | rpl::start_with_next([=] {
+	) | rpl::on_next([=] {
 		updateContentGeometry();
 	}, _content->lifetime());
 }
@@ -784,6 +810,36 @@ void WrapWidget::showFinishedHook() {
 	_bottomShadow->toggle(_bottomShadow->toggled(), anim::type::instant);
 	_topBarSurrogate.destroy();
 	_content->showFinished();
+
+	if (_topBarMenuToggle
+		&& _controller->section().type() == Section::Type::Settings) {
+		const auto controller = _controller->parentController();
+		const auto settingsType = _controller->section().settingsType();
+		const auto highlightId = [&]() -> QString {
+			if (settingsType == ::Settings::MainId()) {
+				return u"settings/log-out"_q;
+			} else if (settingsType == ::Settings::ChatId()) {
+				return u"chat/themes-create"_q;
+			}
+			return QString();
+		}();
+		if (!highlightId.isEmpty()
+			&& controller->takeHighlightControlId(highlightId)) {
+			showTopBarMenu(false);
+			if (_topBarMenu) {
+				const auto menu = _topBarMenu->menu();
+				for (const auto action : menu->actions()) {
+					const auto controlId = "highlight-control-id";
+					if (action->property(controlId).toString() == highlightId) {
+						if (const auto item = menu->itemForAction(action)) {
+							::Settings::HighlightWidget(item);
+						}
+						break;
+					}
+				}
+			}
+		}
+	}
 }
 
 bool WrapWidget::showInternal(
